@@ -27,6 +27,7 @@
 #include "config.h"
 #include <string.h>
 
+#include "json_compatibility.h"
 #include "liblognorm.h"
 #include "lognorm.h"
 #include "annot.h"
@@ -46,6 +47,17 @@ ln_version(void)
 	return VERSION;
 }
 
+ln_ctx
+ln_inherittedCtx(ln_ctx parent)
+{
+	ln_ctx child = ln_initCtx();
+	if (child != NULL) {
+		child->allowRegex = parent->allowRegex;
+		child->dbgCB = parent->dbgCB;
+		child->dbgCookie = parent->dbgCookie;
+	}
+	return child;
+}
 
 ln_ctx
 ln_initCtx(void)
@@ -56,6 +68,7 @@ ln_initCtx(void)
 
 	ctx->objID = LN_ObjID_CTX;
 	ctx->dbgCB = NULL;
+	ctx->allowRegex = 0;
 
 	/* we add an root for the empty word, this simplifies parse
 	 * tree handling.
@@ -77,6 +90,11 @@ done:
 	return ctx;
 }
 
+void
+ln_setCtxOpts(ln_ctx ctx, int allow_regex) {
+	ctx->allowRegex = allow_regex;
+}
+
 
 int
 ln_exitCtx(ln_ctx ctx)
