@@ -32,11 +32,13 @@
  *
  * A copy of the LGPL v2.1 can be found in the file "COPYING" in this distribution.
  */
+#include "config.h"
 #include <stdio.h>
 #include <string.h>
 #include <getopt.h>
 #include <libestr.h>
-#include <json.h>
+
+#include "json_compatibility.h"
 #include "liblognorm.h"
 #include "ptree.h"
 #include "lognorm.h"
@@ -60,10 +62,9 @@ dbgCallBack(void __attribute__((unused)) *cookie, const char *msg,
 	printf("liblognorm: %s\n", msg);
 }
 
-void errout(char *errmsg)
+void complain(const char *errmsg)
 {
 	fprintf(stderr, "%s\n", errmsg);
-	exit(1);
 }
 
 
@@ -185,20 +186,54 @@ genDOT()
 	fwrite(es_getBufAddr(str), 1, es_strlen(str), fpDOT);
 }
 
+static void
+handle_generic_option(const char* opt) {
+	if (strcmp("allowRegex", opt) == 0) {
+		ln_setCtxOpts(ctx, 1);
+	}
+}
+
+static void usage(void)
+{
+fprintf(stderr,
+	"Options:\n"
+	"    -r<rulebase> Rulebase to use. This is required option\n"
+	"    -e<json|xml|csv>\n"
+	"                 Change output format. By default, Mitre CEE is used\n"
+	"    -E<format>   Encoder-specific format (used for CSV, read docs)\n"
+	"    -T           Include 'event.tags' in JSON format\n"
+	"    -oallowRegex Allow regexp matching (read docs about performance penalty)\n"
+	"    -p           Print back only if the message has been parsed succesfully\n"
+	"    -t<tag>      Print back only messages matching the tag\n"
+	"    -v           Print debug. When used 3 times, prints parse tree\n"
+	"    -d           Print DOT file to stdout and exit\n"
+	"    -d<filename> Save DOT file to the filename\n"
+	"\n"
+	);
+}
 
 int main(int argc, char *argv[])
 {
 	int opt;
 	char *repository = NULL;
+	int ret = 0;
+
+	if((ctx = ln_initCtx()) == NULL) {
+		complain("Could not initialize liblognorm context");
+		ret = 1;
+		goto exit;
+	}
 	
-	while((opt = getopt(argc, argv, "d:e:r:E:vpt:T")) != -1) {
+	while((opt = getopt(argc, argv, "d:e:r:E:vpt:To:h")) != -1) {
 		switch (opt) {
 		case 'd': /* generate DOT file */
 			if(!strcmp(optarg, "")) {
 				fpDOT = stdout;
 			} else {
 				if((fpDOT = fopen(optarg, "w")) == NULL) {
-					errout("cannot open DOT file");
+					complain("Cannot open DOT file");
+					ret = 1;
+					goto exit;
 				}
 			}
 		case 'v':
@@ -229,15 +264,21 @@ int main(int argc, char *argv[])
 			     are output */
 			mandatoryTag = es_newStrFromCStr(optarg, strlen(optarg));
 			break;
+		case 'o':
+			handle_generic_option(optarg);
+			break;
+		case 'h':
+			usage();
+			ret = 1;
+			goto exit;
+			break;
 		}
 	}
 	
 	if(repository == NULL) {
-		errout("samples repository must be given");
-	}
-
-	if((ctx = ln_initCtx()) == NULL) {
-		errout("Could not initialize liblognorm context");
+		complain("Samples repository must be given (-r)");
+		ret = 1;
+		goto exit;
 	}
 
 	if(verbose) {
@@ -252,15 +293,17 @@ int main(int argc, char *argv[])
 
 	if(fpDOT != NULL) {
 		genDOT();
-		exit(1);
+		ret=1;
+		goto exit;
 	}
 
 	if(verbose > 2) ln_displayPTree(ctx->ptree, 0);
 
 	normalize();
 
-	ln_exitCtx(ctx);
+exit:
+	if (ctx) ln_exitCtx(ctx);
 	if (encFmt != NULL)
 		free(encFmt);
-	return 0;
+	return ret;
 }
