@@ -33,8 +33,8 @@
 #include <assert.h>
 #include <ctype.h>
 #include <libestr.h>
-#include <json.h>
 
+#include "json_compatibility.h"
 #include "liblognorm.h"
 #include "lognorm.h"
 #include "samp.h"
@@ -69,13 +69,17 @@ ln_newPTree(ln_ctx ctx, struct ln_ptree **parentptr)
 done:	return tree;
 }
 
-static void
+void
 ln_deletePTreeNode(ln_fieldList_t *node)
 {
 	ln_deletePTree(node->subtree);
 	es_deleteStr(node->name);
 	if(node->data != NULL)
 		es_deleteStr(node->data);
+	if(node->raw_data != NULL)
+		es_deleteStr(node->raw_data);
+	if(node->parser_data != NULL && node->parser_data_destructor != NULL)
+		node->parser_data_destructor(&(node->parser_data));
 	free(node);
 }
 
@@ -373,9 +377,9 @@ ln_addFDescrToPTree(struct ln_ptree **tree, ln_fieldList_t *node)
 	for(curr = (*tree)->froot ; curr != NULL ; curr = curr->next) {
 		if(!es_strcmp(curr->name, node->name) 
 				&& curr->parser == node->parser
-				&& ((curr->data == NULL && node->data == NULL)
-					|| (curr->data != NULL && node->data != NULL
-						&& !es_strcmp(curr->data, node->data)))) {
+				&& ((curr->raw_data == NULL && node->raw_data == NULL)
+					|| (curr->raw_data != NULL && node->raw_data != NULL
+						&& !es_strcmp(curr->raw_data, node->raw_data)))) {
 			*tree = curr->subtree;
 			ln_deletePTreeNode(node);
 			r = 0;
@@ -534,13 +538,13 @@ addUnparsedField(const char *str, size_t strLen, int offs, struct json_object *j
 	if (value == NULL) {
 		goto done;
 	}
-	json_object_object_add(json, "originalmsg", value);
+	json_object_object_add(json, ORIGINAL_MSG_KEY, value);
 	
 	value = json_object_new_string(s + offs);
 	if (value == NULL) {
 		goto done;
 	}
-	json_object_object_add(json, "unparsed-data", value);
+	json_object_object_add(json, UNPARSED_DATA_KEY, value);
 
 	r = 0;
 done:
@@ -716,7 +720,7 @@ ln_normalizeRec(struct ln_ptree *tree, const char *str, size_t strLen, size_t of
 			}
 		} else {
 			value = NULL;
-			localR = node->parser(str, strLen, &i, node->data, &parsed, &value);
+			localR = node->parser(str, strLen, &i, node, &parsed, &value);
 			ln_dbgprintf(tree->ctx, "parser returns %d, parsed %zu", localR, parsed);
 			if(localR == 0) {
 				/* potential hit, need to verify */
