@@ -1,6 +1,6 @@
 /* samp.c -- code for ln_samp objects.
  *
- * Copyright 2010 by Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2010-2015 by Rainer Gerhards and Adiscon GmbH.
  *
  * Modified by Pavel Levshin (pavel@levshin.spb.ru) in 2013
  *
@@ -30,6 +30,7 @@
 #include <assert.h>
 #include <ctype.h>
 
+#include "json_compatibility.h"
 #include "liblognorm.h"
 #include "lognorm.h"
 #include "samp.h"
@@ -105,26 +106,40 @@ ln_sampFree(ln_ctx __attribute__((unused)) ctx, struct ln_samp *samp)
  * @returns 0 on success, something else otherwise
  */
 static inline int
-parseFieldDescr(ln_ctx ctx, struct ln_ptree **subtree, es_str_t *rule,
+addFieldDescr(ln_ctx ctx, struct ln_ptree **subtree, es_str_t *rule,
 	        es_size_t *bufOffs, es_str_t **str)
 {
 	int r;
+	ln_fieldList_t *node = ln_parseFieldDescr(ctx, rule, bufOffs, str, &r);
+	assert(subtree != NULL);
+
+	if (node != NULL) CHKR(ln_addFDescrToPTree(subtree, node));
+done:
+	return r;
+}
+
+ln_fieldList_t*
+ln_parseFieldDescr(ln_ctx ctx, es_str_t *rule, es_size_t *bufOffs, es_str_t **str, int* ret)
+{
+	int r = 0;
 	ln_fieldList_t *node;
 	es_size_t i = *bufOffs;
 	char *cstr;	/* for debug mode strings */
 	unsigned char *buf;
 	es_size_t lenBuf;
-
-	assert(subtree != NULL);
+	void* (*constructor_fn)(ln_fieldList_t *, ln_ctx) = NULL;
 
 	buf = es_getBufAddr(rule);
 	lenBuf = es_strlen(rule);
 	assert(buf[i] == '%');
 	++i;	/* "eat" ':' */
-	CHKN(node = malloc(sizeof(ln_fieldList_t)));
+	CHKN(node = calloc(1, sizeof(ln_fieldList_t)));
 	node->subtree = NULL;
 	node->next = NULL;
 	node->data = NULL;
+	node->raw_data = NULL;
+	node->parser_data = NULL;
+	node->parser_data_destructor = NULL;
 	CHKN(node->name = es_newStr(16));
 
 	while(i < lenBuf && buf[i] != ':') {
@@ -168,12 +183,20 @@ parseFieldDescr(ln_ctx ctx, struct ln_ptree **subtree, es_str_t *rule,
 		node->parser = ln_parseRFC5424Date;
 	} else if(!es_strconstcmp(*str, "number")) {
 		node->parser = ln_parseNumber;
+	} else if(!es_strconstcmp(*str, "float")) {
+		node->parser = ln_parseFloat;
+	} else if(!es_strconstcmp(*str, "hexnumber")) {
+		node->parser = ln_parseHexNumber;
 	} else if(!es_strconstcmp(*str, "ipv4")) {
 		node->parser = ln_parseIPv4;
 	} else if(!es_strconstcmp(*str, "word")) {
 		node->parser = ln_parseWord;
+	} else if(!es_strconstcmp(*str, "alpha")) {
+		node->parser = ln_parseAlpha;
 	} else if(!es_strconstcmp(*str, "rest")) {
 		node->parser = ln_parseRest;
+	} else if(!es_strconstcmp(*str, "op-quoted-string")) {
+		node->parser = ln_parseOpQuotedString;
 	} else if(!es_strconstcmp(*str, "quoted-string")) {
 		node->parser = ln_parseQuotedString;
 	} else if(!es_strconstcmp(*str, "date-iso")) {
@@ -185,13 +208,48 @@ parseFieldDescr(ln_ctx ctx, struct ln_ptree **subtree, es_str_t *rule,
 	} else if(!es_strconstcmp(*str, "iptables")) {
 		node->parser = NULL;
 		node->isIPTables = 1;
+	} else if(!es_strconstcmp(*str, "string-to")) {
+		// TODO: check extra data!!!! (very important)
+		node->parser = ln_parseStringTo;
 	} else if(!es_strconstcmp(*str, "char-to")) {
 		// TODO: check extra data!!!! (very important)
 		node->parser = ln_parseCharTo;
 	} else if(!es_strconstcmp(*str, "char-sep")) {
 		// TODO: check extra data!!!! (very important)
 		node->parser = ln_parseCharSeparated;
-	} else {
+	} else if(!es_strconstcmp(*str, "tokenized")) {
+		node->parser = ln_parseTokenized;
+		constructor_fn = tokenized_parser_data_constructor;
+		node->parser_data_destructor = tokenized_parser_data_destructor;
+	}
+#ifdef FEATURE_REGEXP
+	else if(!es_strconstcmp(*str, "regex")) {
+		node->parser = ln_parseRegex;
+		constructor_fn = regex_parser_data_constructor;
+		node->parser_data_destructor = regex_parser_data_destructor;
+	}
+#endif
+    else if (!es_strconstcmp(*str, "recursive")) {
+        node->parser = ln_parseRecursive;
+        constructor_fn = recursive_parser_data_constructor;
+        node->parser_data_destructor = recursive_parser_data_destructor;
+    } else if (!es_strconstcmp(*str, "descent")) {
+        node->parser = ln_parseRecursive;
+        constructor_fn = descent_parser_data_constructor;
+        node->parser_data_destructor = recursive_parser_data_destructor;
+    } else if (!es_strconstcmp(*str, "interpret")) {
+        node->parser = ln_parseInterpret;
+        constructor_fn = interpret_parser_data_constructor;
+        node->parser_data_destructor = interpret_parser_data_destructor;
+    } else if (!es_strconstcmp(*str, "suffixed")) {
+        node->parser = ln_parseSuffixed;
+        constructor_fn = suffixed_parser_data_constructor;
+        node->parser_data_destructor = suffixed_parser_data_destructor;
+    } else if (!es_strconstcmp(*str, "named_suffixed")) {
+        node->parser = ln_parseSuffixed;
+        constructor_fn = named_suffixed_parser_data_constructor;
+        node->parser_data_destructor = suffixed_parser_data_destructor;
+    } else {
 		cstr = es_str2cstr(*str, NULL);
 		ln_dbgprintf(ctx, "ERROR: invalid field type '%s'", cstr);
 		free(cstr);
@@ -211,6 +269,7 @@ parseFieldDescr(ln_ctx ctx, struct ln_ptree **subtree, es_str_t *rule,
 			}
 			CHKR(es_addChar(&node->data, buf[i++]));
 		}
+		node->raw_data = es_strdup(node->data);
 		es_unescapeStr(node->data);
 		if(ctx->debug) {
 			cstr = es_str2cstr(node->data, NULL);
@@ -219,16 +278,20 @@ parseFieldDescr(ln_ctx ctx, struct ln_ptree **subtree, es_str_t *rule,
 		}
 	}
 
+	if (constructor_fn) node->parser_data = constructor_fn(node, ctx);
+
 
-	/* finished */
-	CHKR(ln_addFDescrToPTree(subtree, node));
 	*bufOffs = i;
-	r = 0;
-
-done:	return r;
+done:
+	if (r != 0) {
+		if (node->name != NULL) es_deleteStr(node->name);
+		free(node);
+		node = NULL;
+	}
+	*ret = r;
+	return node;
 }
 
-
 /**
  * Parse a Literal string out of the template and add it to the tree.
  * @param[in] ctx the context
@@ -259,7 +322,7 @@ parseLiteral(ln_ctx ctx, struct ln_ptree **subtree, es_str_t *rule,
 			if(i+1 < lenBuf && buf[i+1] != '%') {
 				break; /* field start is end of literal */
 			}
-			++i;
+			if (++i == lenBuf) break;
 		}
 		CHKR(es_addChar(str, buf[i]));
 		++i;
@@ -310,7 +373,7 @@ addSampToTree(ln_ctx ctx, es_str_t *rule, struct json_object *tagBucket)
 		CHKR(parseLiteral(ctx, &subtree, rule, &i, &str));
 		/* After the literal there can be field only*/
 		if (i < es_strlen(rule)) {
-			CHKR(parseFieldDescr(ctx, &subtree, rule, &i, &str));
+			CHKR(addFieldDescr(ctx, &subtree, rule, &i, &str));
 			if (i == es_strlen(rule)) {
 				/* finish the tree with empty literal to avoid false merging*/
 				CHKR(parseLiteral(ctx, &subtree, rule, &i, &str));
